--[[
 Task Queues!
]]--
math.randomseed( os.time() )
math.random(); math.random(); math.random()


local kernel01 = {
	"assembler",
}

local kernel02 = {
	"bit",
}

local kernel03 = {
	"byte",
}

local kernel04 = {
	"pointer",
}

local kernel05 = {
	"bit",
	"bit",
	"bit",
	"assembler",
	"byte",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}
local kernel06={
	"assembler",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
}
local kernel07={
	"bit",
	"bit",
	"bit",
	"assembler",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}
local kernel08={
	"assembler",
	"bit",
	"bit",
	"bit",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}
local kernel09={
	"assembler",
	"assembler",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}
local kernel10={
	"bit",
	"bit",
	"bit",
	"assembler",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
}
local kernel11={
	"bit",
	"assembler",
	"bit",
	"bit",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}
local kernel12={
	"assembler",
	"bit",
	"bit",
	"bit",
	"assembler",
	"byte",
	"byte",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
}
local kernel13 = {
	"byte",
	"assembler",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"assembler",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
}

local kernel14={
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"assembler",
	"byte",
	"byte",
	"byte",
	"pointer",
	"byte",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"assembler",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
	"pointer",
	"byte",
}

local hole01={
	"trojan",
}
local hole02={
	"bug",
}
local hole03={
	"worm",
}
local hole04={
	"dos",
}
local hole05={
	"bug",
	"bug",
	"bug",
	"trojan",
	"worm",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole06={
	"trojan",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
}
local hole07={
	"bug",
	"bug",
	"bug",
	"trojan",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole08={
	"trojan",
	"bug",
	"bug",
	"bug",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole09={
	"trojan",
	"trojan",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole10={
	"bug",
	"bug",
	"bug",
	"trojan",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
}
local hole11={
	"bug",
	"trojan",
	"bug",
	"bug",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole12={
	"trojan",
	"bug",
	"bug",
	"bug",
	"trojan",
	"worm",
	"worm",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
}
local hole13={
	"worm",
	"trojan",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"trojan",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
}
local hole14={
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"trojan",
	"worm",
	"worm",
	"worm",
	"dos",
	"worm",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm" ,
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"trojan",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
	"dos",
	"worm",
}

taskqueues = {
	kernel = (
		function()
			local r = math.random(0,14)
			if r == 0 then
				return kernel01
			elseif r == 1 then
				return kernel02
			elseif r == 2 then
				return kernel03
			elseif r == 3 then
				return kernel04
			elseif r == 4 then
				return kernel05
			elseif r == 5 then
				return kernel06
			elseif r == 6 then
				return kernel07
			elseif r == 7 then
				return kernel08
			elseif r == 8 then
				return kernel09
			elseif r == 9 then
				return kernel10
			elseif r == 10 then
				return kernel11
			elseif r == 11 then
				return kernel12
			elseif r == 12 then
				return kernel13
			else
				return kernel14
			end
		end
		),
	assembler = {
		"socket",
	},
	socket = {
		"bit",
	},
	window = {
		"bug",
	},
	trojan = {
		"window",
	},
	hole = (
		function()
			local r = math.random(0,14)
			if r == 0 then
				return hole01
			elseif r == 1 then
				return hole02
			elseif r == 2 then
				return hole03
			elseif r == 3 then
				return hole04
			elseif r == 4 then
				return hole05
			elseif r == 5 then
				return hole06
			elseif r == 6 then
				return hole07
			elseif r == 7 then
				return hole08
			elseif r == 8 then
				return hole09
			elseif r == 9 then
				return hole10
			elseif r == 10 then
				return hole11
			elseif r == 11 then
				return hole12
			elseif r == 12 then
				return hole13
			else
				return hole14
			end
		end
		),
}
