shard_include "maphandler"
shard_include "buildsitehandler"
shard_include "counthandler"
shard_include "unithandler"
shard_include "attackhandler"
shard_include "bomberhandler"
shard_include "sleep"
shard_include "raidhandler"
shard_include "loshandler"
shard_include "targethandler"
shard_include "scouthandler"
shard_include "assisthandler"
shard_include "defendhandler"
shard_include "turtlehandler"
shard_include "econhandler"
shard_include "tacticalhandler"
shard_include "damagehandler"
shard_include "cleanhandler"

modules = { Sleep, MapHandler, EconHandler, AttackHandler, BomberHandler, RaidHandler, BuildSiteHandler, CountHandler, TurtleHandler, TacticalHandler, LosHandler, TargetHandler, DamageHandler, ScoutHandler, AssistHandler, DefendHandler, CleanHandler, UnitHandler }
