--[[
 Task Queues!
]]--

local armcommander = {
	"armlab",
	"armmex",
	"armmex",
	"armmex",
	"mexpylon",
	"armlab",
	"armsolar",
	"armsolar",
	"armrl",
	"armrl",
	"armrl",
	"armnanotc",
	"armrad",
	"armwin",
	"armwin",
	"armwin",
	--	"mexpylon", --Moved this.
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"pylon",
	"armwin",
	"armwin",
	"armsolar",
	"armsolar",
	"armvp",
	"armsolar",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"armnanotc",
	"pylon",
	"armjamt",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armrl",
	"pylon",
	"armap",
	"armhlt",
	"armllt",
	"armrl",
	"armjamt",
	"armmex",
	"armmex",
	"armmex",
	"armsolar",
	"armsolar",
	"armrl",
	"armrl",
	"armrl",
	"armnanotc",
	"armrad",
	"armwin",
	"armwin",
	"armwin",
	"mexpylon",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"pylon",
	"armwin",
	"armwin",
	"armsolar",
	"armsolar",
	"armsolar",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armwin",
	"armwin",
	"armnanotc",
	"pylon",
	"armjamt",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armrl",
	"pylon",
}
taskqueues = {
	armcom = armcommander,
	armlab = {
	"armrectr",
	"armrectr",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armrock",
	"armrock",
	"armrock",
		"armpw",
	"armpw",
	"armpw",
	"armrectr",
	"armjeth",
	"armjeth",
	"armwar",
	"armwar",
	"armham",
	"armham",
	"armpw",
	"armpw",
	"armrock",
	"armjeth",
	"armrock",
	"armpw",
	"armpw",
	"armpw",
	"armrectr",
	"armwar",
	"armwar",
	"armham",
	"armflea",
	"armflea",
	"armham",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armrectr",
	"armwar",
	"armwar",
	"armwar",
	"armwar",
	"armrock",
	"armrock",
	"armham",
	"armham",
	"armpw",
	"armjeth",
	"armjeth",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armjeth",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armrock",
	"armpw",
	"armpw",
	"armpw",
	"armpw",
	"armham",
	"armpw",
	"armpw",
	},
	armap = {
	"armhawk",
	"armfig",
	"armfig",
--	"armpnix", --AI can't use bombers
	"armfig",
	"armfig",
	},
	armrectr = {
	"armmex",
	"armrl",
	"armwin",
	"armwin",
	"armmex",
	"armrl",
	"armrad",
	"armrl",
	"armsolar",
	"armmex",
	"armwin",
	"armsolar",
	"armllt",
	"armrl",
	"armrl",
	"armrl",
	"armarch",
	"armllt",
	"armhlt",
	"armjamt",
	"armrl",
	"armpb",
	"armwin",
	"mexpylon",
	"armsolar",
	"armsolar",
	"armsolar",
	"armrl",
	"armrl",
	"armnanotc",
	"armvp",
	"armrl",
	"armrl",
	"armhlt",
	"armwin",
	"armwin",
	"armwin",
	"armwin",
	"armrl",
	"armrl",
	"armarch",
	"armlab",

	},
	armvp = {
	"pioneer",
	"armfav",
	"armfav",
	"armfav",
	"armfav",
	"armflash",
	"armflash",
	"armflash",
	"armflash",
	"armstump",
	"armstump",
	"armjanus",
	"armstump",
	"armstump",
	"armstump",
	"armjanus",
	"armjanus",
	"armflash",
	"armflash",
	"TAWF013", 
	"armstump",
	"armflash",
	"TAWF013",
	"armfav",
	"armfav",
	"armfav",
	},
	pioneer = {
	"armwin",
	"armwin",
	"armrl",
	"armhlt",
	"armfus",
	"pylon",
	"armarch",
	"armsolar",
	"armrl",
	"armpb",
	"armwin",
	"armmstore",
	"armmstore",
	},
}