shard_include( "spothandler" )
shard_include( "controlpointhandler" )
shard_include( "unithandler" )
shard_include( "attackhandler" )
shard_include( "sleep" )

modules = { UnitHandler, AttackHandler, ControlPointHandler, MetalSpotHandler, Sleep }
